{=====================================================================}
{ PURPOSE : Checks a list of text files for a specific ASCII string,  }
{           entered by the user.                                      }
{=====================================================================}
{ SYSTEM  : MS-DOS / Turbo Pascal                                     }
{                      Ver. 7.0                                       }
{                                                                     }
{ Source : SEARCH.PAS                                                 }
{                                                                     }
{ Author: M.Genovy                                                    }
{                                                                     }
{ Date: January 15, 2000                                              }
{                                                                     }
{ Last Modification :                                                 }
{                                                                     }
{                                                                     }
{=====================================================================}

{---------------------------------------------------------------------}
{ OPERATION:                                                          }
{ ---------                                                           }
{ Prompts for the name of the file containing a list of text files to }
{ be searched. This program reads the name of a text file from this   }
{ list file and then opens the text file and searches for the user    }
{ specified ASCII string.                                             }
{                                                                     }
{---------------------------------------------------------------------}

program SEARCH;

uses
  CRT,
  DOS,
  Printer;

CONST

Title    = 'SEARCH - Search text files for ASCII string';
Version  = 'Version 1.00';

Blank = #32; { ASCII Blank Character }

VAR

   InFile1        : TEXT;        { Input data file # 1 type           }
   InFile2        : TEXT;        { Input data file # 2 type           }
   OutFile        : TEXT;        { Output data file type              }
   OutFile2       : TEXT;        { Error Log data file type           }
   Line1          : string[255]; { Line from input data file # 1      }
   Line2          : string[255]; { Line from input data file # 2      }
   SearchString   : string[255]; { User entered string to search for  }
   TempLine       : string;      { Tempoary line storage string       }
   i              : integer;     { Loop index                         }
   Blanks         : integer;
   FileCount1     : integer; { Number of text files searched          }
   LineCount2     : integer; { Number of lines in Input data file 2   }
   TotalCount     : LongInt; { Total number of source code lines      }
                             { scanned                                }
   SourceCount    : LongInt; { Total number of SOURCE files with prob }
   ProblemCount   : LongInt; { Total number of lines with a problem   }
   OutLine        : string;  { Output file line                       }
                             { end reached                            }

  InFileName1     : string;  { Input file # 1 name                    }
  InFileName2     : string;  { Input file # 2 name                    }
  OutFileName     : string;  { Output file name                       }
  FileMissing     : boolean; { Flag to indicate source file missing   }
  FirstPass       : boolean; { Flag to indicate first pass thru file  }

{ System time and date }

  Year,Day,Month,DayOfWeek      : Word;
  { System time at the start of the schedule }
  Hour,Minute,Second,Sec100     : Word;

{=====================================================================}
{  Main Routine                                                       }
{=====================================================================}
{                                                                     }
{ Input File Format                                                   }
{ ------------------                                                  }
{                                                                     }
{ A typical input file is shown below.                                }
{                                                                     }
{ 12345678.123              Input File # 1  ( ie. 8.3 format )        }
{ textfile.txt              Input File # 2                            }
{ mytxtfil.txt              Input File # 3                            }
{                                                                     }
{---------------------------------------------------------------------}

begin { Main program }

{ Set up default file names }

  InFilename1 := '';
  InFileName2 := '';
  OutFilename := '';

  WriteLn;
  WriteLn(Title,' - ', Version);
  WriteLn;

  WriteLn('File with text file name list ?');
  ReadLn(InFileName1);

  WriteLn('Output File name where details of search to be logged ?');
  ReadLn(OutFileName);

  WriteLn('What string do you want to search for ?');
  ReadLn(SearchString);

{ Check that file names were actually entered }

  If (InfileName1 = '') or (OutFilename = '') then
  begin
    WriteLn('File names not entered correctly. Exiting....');
    exit;
  end;

{=====================================================================}
{ Check for existence of the Input data file. If no file exists abort }
{ Open files for Read access only.                                    }
{=====================================================================}

{ Assign the input data file # 1 name }

   Assign(InFile1,InFileName1);

{ See if the data file exists - if not the abort }

{$i-}

   Reset(InFile1);	{ Open an existing file }

{$i+}

   If IoResult <> 0 then
   begin
     WriteLn;
     WriteLn('Input data file # 1 ',InFileName1,' does not exist....aborting');
     Delay(1000);
     exit;
   end;

{=====================================================================}
{ Create a new output file each time this program is run.             }
{=====================================================================}

{ Assign the Output data file name }

   Assign(OutFile,OutFileName);

   Rewrite(OutFile);

{ Get the date and time for the transfer log file }

  GetDate(Year, Month, Day , DayofWeek);
  GetTime(Hour, Minute, Second, Sec100);

{=====================================================================}
{ Create a log file header                                            }
{=====================================================================}

   WriteLn(OutFile,'==========================================================================');
   WriteLn(OutFile,'                           SEARCH.EXE');
   WriteLn(OutFile,'==========================================================================');
   WriteLn(OutFile,' Date: ',Month,'/',Day,'/',Year,'                                    Time: ',Hour,':',Minute,':',Second);
   WriteLn(OutFile,' ');
   WriteLn(OutFile,'  Input file name is           - ',InFilename1);
   WriteLn(OutFile,'  Output file name is          - ',OutFilename);
   WriteLn(OutFile,'  String searched for is         |',SearchString,'|');
   WriteLn(OutFile,' ');
   WriteLn(OutFile,'==========================================================================');
   WriteLn(OutFile,' ');


{=====================================================================}
{ Now read a line from source name file and open the source file      }
{=====================================================================}

   TotalCount := 0;
   SourceCount := 0;
   ProblemCount := 0;
   FileCount1 := 0;
   FileMissing := False;
   FirstPass := True;           { Set flag to indicate first pass thru }
                                { this source file                     }

   While Not Eof(InFile1)  do
   begin

     ReadLn(InFile1,Line1);
     FileCount1 := FileCount1 + 1;      { Bump up file counter }

{ Parse out the source file name }

     If Pos(' ',Line1) > 0 then
       InFileName2 := Copy(Line1,1,(Pos(' ',Line1)-1))
     else
       InFileName2 := Line1;

{ Assign the input data file # 2 name }

     Assign(InFile2,InFileName2);

{ See if the data file exists - if not then abort }

{$i-}

     Reset(InFile2);	{ Open an existing file }

{$i+}

     If IoResult <> 0 then
     begin
       WriteLn;
       WriteLn('*** Text file ',InFileName2,' does not exist.');
       WriteLn(OutFile,' *** Text file ',InFilename2,' does not exist.');
       WriteLn(OutFile,' ');
{ Set flag to skip this file }
       FileMissing := True;
     end;

   If Not FileMissing then
   begin

     WriteLn('Scanning Text File # ',FileCount1,'  Name = ',InfileName2);

     While Not Eof(InFile2) do
     begin

{ Now read in line from source file and check for search string }

       ReadLn(InFile2,Line2);
       LineCount2 := LineCount2 + 1;

       If Pos(SearchString,Line2) > 0 then
       begin
         WriteLn('Text File ',InfileName2,' line # ',LineCount2,' string found. ');
         WriteLn(' ',Line2);
         If FirstPass then
         begin
           WriteLn(OutFile,'--------------------------------------------------------------------------');
           WriteLn(OutFile,'Text File name = ',InFilename2);
           WriteLn(OutFile,'--------------------------------------------------------------------------');
           SourceCount := SourceCount + 1;
           FirstPass := False;
         end;
         WriteLn(OutFile,' Line # ',LineCount2,' string found. ');
         WriteLn(OutFile,' ',Line2);
         WriteLn(OutFile,' ');
         ProblemCount := ProblemCount + 1;
       end;

     end;

{ Close the source file }

    Close(Infile2);

   end;

{ Reset the counters, clear FileMissing flag, and set FirstPass flag }

   TotalCount := LineCount2 + TotalCount;
   LineCount2 := 0;
   FileMissing := False;
   FirstPass := True;

{=====================================================================}
{ Loop back and process the next source file                          }
{=====================================================================}

   end; { do }

{=====================================================================}
{ Write a summary to the output file.                                 }
{=====================================================================}

   WriteLn(OutFile,'==========================================================================');
   WriteLn(OutFile,'                           SEARCH SUMMARY');
   WriteLn(OutFile,'==========================================================================');
   WriteLn(OutFile,' Total number of text files scanned                    = ',FileCount1);
   WriteLn(OutFile,' Total number of text files with specified string      = ',SourceCount);
   WriteLn(OutFile,' Total number of text lines scanned                    = ',TotalCount);
   WriteLn(OutFile,' Total number of text lines with specified string      = ',ProblemCount);
   WriteLn(OutFile,'==========================================================================');

{=====================================================================}
{ Close all files and exit                                            }
{=====================================================================}

     Close(InFile1);    { The file with text file names           }
     Close(OutFile);    { The log file with search results listed }

END.